/*
 * XdccDownloader - Command.java - Copyright © 2013 David Roden
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.pterodactylus.xdcc.ui.stdin;

import java.io.IOException;
import java.io.Writer;
import java.util.Collection;
import java.util.List;

import com.google.common.base.Function;

/**
 * A command is executed by the {@link CommandReader}. It receives the current
 * state of the command reader and can return a changed state, depending on the
 * given parameters.
 *
 * @author <a href="mailto:bombe@pterodactylus.net">David ‘Bombe’ Roden</a>
 */
public interface Command {

	/** Converts a command into its name. */
	public static final Function<Command, String> TO_NAME = new Function<Command, String>() {
		@Override
		public String apply(Command command) {
			return command.getName();
		}
	};

	/** Converts a command into its aliases. */
	public static final Function<Command, Collection<String>> TO_ALIASES = new Function<Command, Collection<String>>() {
		@Override
		public Collection<String> apply(Command command) {
			return command.getAliases();
		}
	};

	/**
	 * Returns the name of this command.
	 *
	 * @return The name of this command
	 */
	public String getName();

	/**
	 * Returns possible aliases for this command.
	 *
	 * @return Possible aliases for this command
	 */
	public Collection<String> getAliases();

	/**
	 * Executes this command.
	 *
	 * @param state
	 * 		The current state of the command reader
	 * @param parameters
	 * 		The parameters given on the command line
	 * @param outputWriter
	 * 		The output writer to write the output of the command to
	 * @return The new state of the command reader (which may be the given state if
	 *         the command does not need to modify the state)
	 * @throws IOException
	 * 		if an I/O error occurs
	 */
	public State execute(State state, List<String> parameters, Writer outputWriter) throws IOException;

}
